@@ -4,13 +4,14 @@
 /// if the query has output columns, or `()` (unit) otherwise:
 ///
 /// ```rust
+/// # use sqlx::Connect;
 /// # #[cfg(all(feature = "mysql", feature = "runtime-async-std"))]
 /// # #[async_std::main]
 /// # async fn main() -> sqlx::Result<()>{
 /// # let db_url = dotenv::var("DATABASE_URL").expect("DATABASE_URL must be set");
 /// #
 /// # if !(db_url.starts_with("mysql") || db_url.starts_with("mariadb")) { return Ok(()) }
-/// # let mut conn = sqlx::MySqlConnection::open(db_url).await?;
+/// # let mut conn = sqlx::MySqlConnection::connect(db_url).await?;
 /// // let mut conn = <impl sqlx::Executor>;
 /// let account = sqlx::query!("select (1) as id, 'Herp Derpinson' as name")
 ///     .fetch_one(&mut conn)
@@ -32,13 +33,14 @@
 /// and this macro will typecheck passed arguments and error on missing ones:
 ///
 /// ```rust
+/// # use sqlx::Connect;
 /// # #[cfg(all(feature = "mysql", feature = "runtime-async-std"))]
 /// # #[async_std::main]
 /// # async fn main() -> sqlx::Result<()>{
 /// # let db_url = dotenv::var("DATABASE_URL").expect("DATABASE_URL must be set");
 /// #
 /// # if !(db_url.starts_with("mysql") || db_url.starts_with("mariadb")) { return Ok(()) }
-/// # let mut conn = sqlx::mysql::MySqlConnection::open(db_url).await?;
+/// # let mut conn = sqlx::mysql::MySqlConnection::connect(db_url).await?;
 /// // let mut conn = <impl sqlx::Executor>;
 /// let account = sqlx::query!(
 ///         // just pretend "accounts" is a real table
@@ -62,6 +64,36 @@
 /// * Postgres: `$N` where `N` is the 1-based positional argument index
 /// * MySQL: `?` which matches arguments in order that it appears in the query
 ///
+/// ## Nullability: Bind Parameters
+/// For a given expected type `T`, both `T` and `Option<T>` are allowed (as well as either
+/// behind references). `Option::None` will be bound as `NULL`, so if binding a type behind `Option`
+/// be sure your query can support it.
+///
+/// Note, however, if binding in a `where` clause, that equality comparisons with `NULL` may not
+/// work as expected; instead you must use `IS NOT NULL` or `IS NULL` to check if a column is not
+/// null or is null, respectively. Note that `IS [NOT] NULL` cannot be bound as a parameter either;
+/// you must modify your query string instead.
+///
+/// ## Nullability: Output Columns
+/// In most cases, the database engine can tell us whether or not a column may be `NULL`, and
+/// the `query!()` macro adjusts the field types of the returned struct accordingly.
+///
+/// For Postgres and SQLite, this only works for columns which come directly from actual tables,
+/// as the implementation will need to query the table metadata to find if a given column
+/// has a `NOT NULL` constraint. Columns that do not have a `NOT NULL` constraint or are the result
+/// of an expression are assumed to be nullable and so `Option<T>` is used instead of `T`.
+///
+/// For MySQL, the implementation looks at [the `NOT_NULL` flag](https://dev.mysql.com/doc/dev/mysql-server/8.0.12/group__group__cs__column__definition__flags.html#ga50377f5ca5b3e92f3931a81fe7b44043)
+/// of [the `ColumnDefinition` structure in `COM_QUERY_OK`](https://dev.mysql.com/doc/internals/en/com-query-response.html#column-definition):
+/// if it is set, `T` is used; if it is not set, `Option<T>` is used.
+///
+/// MySQL appears to be capable of determining the nullability of a result column even if it
+/// is the result of an expression, depending on if the expression may in any case result in
+/// `NULL` which then depends on the semantics of what functions are used. Consult the MySQL
+/// manual for the functions you are using to find the cases in which they return `NULL`.
+///
+/// To override the nullability of an output column, use [query_as!].
+///
 /// ## Requirements
 /// * The `DATABASE_URL` environment variable must be set at build-time to point to a database
 /// server with the schema that the query string will be checked against. (All variants of
@@ -117,22 +149,22 @@ macro_rules! query (
 /// -----
 ///
 /// `examples/queries/account-by-id.sql`:
-/// ```sql
+/// ```text
 /// select * from (select (1) as id, 'Herp Derpinson' as name) accounts
 /// where id = ?
 /// ```
 ///
 /// `src/my_query.rs`:
 /// ```rust
+/// # use sqlx::Connect;
 /// # #[cfg(all(feature = "mysql", feature = "runtime-async-std"))]
 /// # #[async_std::main]
 /// # async fn main() -> sqlx::Result<()>{
 /// # let db_url = dotenv::var("DATABASE_URL").expect("DATABASE_URL must be set");
 /// #
 /// # if !(db_url.starts_with("mysql") || db_url.starts_with("mariadb")) { return Ok(()) }
-/// # let mut conn = sqlx::MySqlConnection::open(db_url).await?;
-/// // let mut conn = <impl sqlx::Executor>;
-/// let account = sqlx::query_file!("examples/queries/account-by-id.sql", 1i32)
+/// # let mut conn = sqlx::MySqlConnection::connect(db_url).await?;
+/// let account = sqlx::query_file!("tests/test-query-account-by-id.sql", 1i32)
 ///     .fetch_one(&mut conn)
 ///     .await?;
 ///
@@ -180,13 +212,14 @@ macro_rules! query_file (
 ///
 /// The only modification to the syntax is that the struct name is given before the SQL string:
 /// ```rust
+/// # use sqlx::Connect;
 /// # #[cfg(all(feature = "mysql", feature = "runtime-async-std"))]
 /// # #[async_std::main]
 /// # async fn main() -> sqlx::Result<()>{
 /// # let db_url = dotenv::var("DATABASE_URL").expect("DATABASE_URL must be set");
 /// #
 /// # if !(db_url.starts_with("mysql") || db_url.starts_with("mariadb")) { return Ok(()) }
-/// # let mut conn = sqlx::MySqlConnection::open(db_url).await?;
+/// # let mut conn = sqlx::MySqlConnection::connect(db_url).await?;
 /// #[derive(Debug)]
 /// struct Account {
 ///     id: i32,
@@ -211,6 +244,10 @@ macro_rules! query_file (
 /// # #[cfg(any(not(feature = "mysql"), not(feature = "runtime-async-std")))]
 /// # fn main() {}
 /// ```
+///
+/// ## Nullability
+/// Use `Option` for columns which may be `NULL` in order to avoid a runtime error being returned
+/// from `.fetch_*()`.
 #[macro_export]
 #[cfg_attr(docsrs, doc(cfg(feature = "macros")))]
 macro_rules! query_as (
@@ -235,13 +272,14 @@ macro_rules! query_as (
 /// Enforces requirements of both macros; see them for details.
 ///
 /// ```rust
+/// # use sqlx::Connect;
 /// # #[cfg(all(feature = "mysql", feature = "runtime-async-std"))]
 /// # #[async_std::main]
 /// # async fn main() -> sqlx::Result<()>{
 /// # let db_url = dotenv::var("DATABASE_URL").expect("DATABASE_URL must be set");
 /// #
 /// # if !(db_url.starts_with("mysql") || db_url.starts_with("mariadb")) { return Ok(()) }
-/// # let mut conn = sqlx::MySqlConnection::open(db_url).await?;
+/// # let mut conn = sqlx::MySqlConnection::connect(db_url).await?;
 /// #[derive(Debug)]
 /// struct Account {
 ///     id: i32,
@@ -249,7 +287,7 @@ macro_rules! query_as (
 /// }
 ///
 /// // let mut conn = <impl sqlx::Executor>;
-/// let account = sqlx::query_file_as!(Account, "examples/queries/account-by-id.sql", 1i32)
+/// let account = sqlx::query_file_as!(Account, "tests/test-query-account-by-id.sql", 1i32)
 ///     .fetch_one(&mut conn)
 ///     .await?;
 ///
@@ -272,7 +310,7 @@ macro_rules! query_file_as (
         }
         macro_result!()
     });
-    ($out_struct:path, $query:literal, $($args:expr),*$(,)?) => (#[allow(dead_code)] {
+    ($out_struct:path, $query:literal, $($args:tt),*$(,)?) => (#[allow(dead_code)] {
         #[macro_use]
         mod _macro_result {
             $crate::sqlx_macros::query_file_as!($out_struct, $query, $($args),*);
@@ -280,3 +318,48 @@ macro_rules! query_file_as (
         macro_result!($($args),*)
     })
 );
+
+/// A variant of [query_as!] which does not check the input or output types. This still does parse
+/// the query to ensure it's syntactically and semantically valid for the current database.
+#[macro_export]
+#[cfg_attr(docsrs, doc(cfg(feature = "macros")))]
+macro_rules! query_as_unchecked (
+    ($out_struct:path, $query:literal) => (#[allow(dead_code)] {
+        #[macro_use]
+        mod _macro_result {
+            $crate::sqlx_macros::query_as_unchecked!($out_struct, $query);
+        }
+        macro_result!()
+    });
+
+    ($out_struct:path, $query:literal, $($args:expr),*$(,)?) => (#[allow(dead_code)] {
+        #[macro_use]
+        mod _macro_result {
+            $crate::sqlx_macros::query_as_unchecked!($out_struct, $query, $($args),*);
+        }
+        macro_result!($($args),*)
+    })
+);
+
+/// A variant of [query_file_as!] which does not check the input or output types. This
+/// still does parse the query to ensure it's syntactically and semantically valid
+/// for the current database.
+#[macro_export]
+#[cfg_attr(docsrs, doc(cfg(feature = "macros")))]
+macro_rules! query_file_as_unchecked (
+    ($out_struct:path, $query:literal) => (#[allow(dead_code)] {
+        #[macro_use]
+        mod _macro_result {
+            $crate::sqlx_macros::query_file_as_unchecked!($out_struct, $query);
+        }
+        macro_result!()
+    });
+
+    ($out_struct:path, $query:literal, $($args:tt),*$(,)?) => (#[allow(dead_code)] {
+        #[macro_use]
+        mod _macro_result {
+            $crate::sqlx_macros::query_file_as_unchecked!($out_struct, $query, $($args),*);
+        }
+        macro_result!($($args),*)
+    })
+);
